@@ -1,6 +1,6 @@
 package Apache::MP3::Resample;
 
-# $Id: Resample.pm,v 1.6 2002/01/06 20:36:09 lstein Exp $
+# $Id: Resample.pm,v 1.2 2002/01/24 23:46:25 lstein Exp $
 # Resamples (downsamples) on the fly
 
 use strict;
@@ -128,7 +128,7 @@ sub open_file {
               {$1 eq 'b' ? $presets         :
                $1 eq 'f' ? quotemeta($file) :
                "%$1"}exg;
-  my $filter = $self->r->dir_config('VerboseMP3Encoder') 
+  my $filter = $self->r->dir_config('VerboseMP3Encoder')
     ? "$encode |" : "$encode 2>/dev/null |";
   return IO::File->new($filter);
 }
@@ -200,6 +200,12 @@ If not present, the following default is used:
 
   lame --mp3input %b - <%f
 
+Some people have reported problems relating to the Apache server's
+path not finding /usr/local/bin/lame.  If this affects you, try the
+following configuration:
+
+ PerlSetVar  MP3Encoder "/usr/local/bin/lame --mp3input %b - <%f"
+
 =item ResamplePresets
 
 A list of bitrates and the command-line options to pass to the
